@@ -537,12 +537,6 @@ describe("printer", function() {
         code = "export function foo() {}";
         ast = parse(code);
 
-        assert.strictEqual(printer.print(ast).code, code);
-        assert.strictEqual(printer.printGenerically(ast).code, code + ";");
-
-        code = "export function foo() {};";
-        ast = parse(code);
-
         assert.strictEqual(printer.print(ast).code, code);
         assert.strictEqual(printer.printGenerically(ast).code, code);
     });
@@ -956,6 +950,26 @@ describe("printer", function() {
         assert.strictEqual(pretty, code);
     });
 
+    it("adds parenthesis around single arrow function arg when options.arrowParensAlways is true", function() {
+      var code = "(a) => {};";
+
+      var fn = b.arrowFunctionExpression(
+          [b.identifier('a')],
+          b.blockStatement([]),
+          false
+      );
+
+      var ast = b.program([
+          b.expressionStatement(fn)
+      ]);
+
+      var printer = new Printer({
+        arrowParensAlways: true
+      });
+      var pretty = printer.printGenerically(ast).code;
+      assert.strictEqual(pretty, code);
+    });
+
     it("adds parenthesis around async arrow functions with args", function() {
         var code = "async () => {};";
 
@@ -1015,6 +1029,73 @@ describe("printer", function() {
         assert.strictEqual(pretty, code);
     });
 
+    it("adds parenthesis around arrow functions with single arg and a return type", function() {
+        var code = "(a): void => {};";
+
+        var arg = b.identifier('a');
+
+        var fn = b.arrowFunctionExpression(
+            [arg],
+            b.blockStatement([]),
+            false
+        );
+
+        fn.returnType = b.typeAnnotation(
+            b.voidTypeAnnotation()
+        );
+
+        var ast = b.program([
+            b.expressionStatement(fn)
+        ]);
+
+        var printer = new Printer();
+        var pretty = printer.printGenerically(ast).code;
+        assert.strictEqual(pretty, code);
+    });
+
+    it("prints class property initializers with type annotations correctly", function() {
+        var code = [
+            "class A {",
+            "  foo = (a: b): void => {};",
+            "}",
+        ].join(eol);
+
+        var arg = b.identifier('a');
+        arg.typeAnnotation = b.typeAnnotation(
+            b.genericTypeAnnotation(b.identifier('b'), null)
+        );
+
+        var fn = b.arrowFunctionExpression(
+            [arg],
+            b.blockStatement([]),
+            false
+        );
+        fn.returnType = b.typeAnnotation(
+            b.voidTypeAnnotation()
+        );
+
+        var ast = b.program([
+            b.classDeclaration(
+                b.identifier('A'),
+                b.classBody([
+                    b.classProperty(
+                        b.identifier('foo'),
+                        fn,
+                        null,
+                        false
+                    )
+                ])
+            )
+        ]);
+
+        var printer = new Printer({
+            tabWidth: 2
+        });
+
+        var pretty = printer.printGenerically(ast).code;
+        assert.strictEqual(pretty, code);
+    });
+
     it("prints ClassProperty correctly", function() {
         var code = [
             "class A {",
@@ -1271,4 +1352,118 @@ describe("printer", function() {
 
         assert.strictEqual(actual, expected);
     });
+
+    it("prints commas for flow object types by default", function() {
+        var code = [
+            "type MyType = {",
+            "    message: string,",
+            "    isAwesome: boolean,",
+            "};"
+        ].join(eol);
+
+        var ast = b.typeAlias(
+            b.identifier("MyType"),
+            null,
+            b.objectTypeAnnotation([
+                b.objectTypeProperty(
+                    b.identifier("message"),
+                    b.stringTypeAnnotation(),
+                    false
+                ),
+                b.objectTypeProperty(
+                    b.identifier("isAwesome"),
+                    b.booleanTypeAnnotation(),
+                    false
+                )
+            ])
+        );
+
+        var printer = new Printer();
+        var pretty = printer.printGenerically(ast).code;
+        assert.strictEqual(pretty, code);
+    });
+
+    it("shouldn't print a trailing comma for single-line flow object types", function() {
+        var code1 = "type MyType = {message: string};";
+        var code2 = "type MyType = {[key: string]: string};";
+
+        var ast1 = b.typeAlias(
+            b.identifier("MyType"),
+            null,
+            b.objectTypeAnnotation([
+                b.objectTypeProperty(
+                    b.identifier("message"),
+                    b.stringTypeAnnotation(),
+                    false
+                )
+            ])
+        );
+
+        var ast2 = b.typeAlias(
+            b.identifier("MyType"),
+            null,
+            b.objectTypeAnnotation([], [
+                b.objectTypeIndexer(
+                    b.identifier('key'),
+                    b.stringTypeAnnotation(),
+                    b.stringTypeAnnotation(),
+                    false
+                )
+            ])
+        );
+
+        var printer = new Printer({trailingComma: true});
+        var pretty1 = printer.printGenerically(ast1).code;
+        var pretty2 = printer.printGenerically(ast2).code;
+        assert.strictEqual(pretty1, code1);
+        assert.strictEqual(pretty2, code2);
+    });
+
+    it("prints semicolons for flow object types when options.flowObjectCommas is falsy", function() {
+        var code = [
+            "type MyType = {",
+            "    message: string;",
+            "    isAwesome: boolean;",
+            "};"
+        ].join(eol);
+
+        var ast = b.typeAlias(
+            b.identifier("MyType"),
+            null,
+            b.objectTypeAnnotation([
+                b.objectTypeProperty(
+                    b.identifier("message"),
+                    b.stringTypeAnnotation(),
+                    false
+                ),
+                b.objectTypeProperty(
+                    b.identifier("isAwesome"),
+                    b.booleanTypeAnnotation(),
+                    false
+                )
+            ])
+        );
+
+        var printer = new Printer({ flowObjectCommas: false });
+        var pretty = printer.printGenerically(ast).code;
+        assert.strictEqual(pretty, code);
+    });
+
+    it("prints parens for nullable union/intersection types", function() {
+        var code = "type MyType = ?(string | number);";
+
+        var ast = b.typeAlias(
+            b.identifier("MyType"),
+            null,
+            b.nullableTypeAnnotation(
+                b.unionTypeAnnotation(
+                    [b.stringTypeAnnotation(), b.numberTypeAnnotation()]
+                )
+            )
+        );
+
+        var printer = new Printer({});
+        var pretty = printer.printGenerically(ast).code;
+        assert.strictEqual(pretty, code);
+    });
 });
